%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%Parameter Estimation in ARX model using Linear Regression%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PART A: This section of the code estimates the model parameters using step
%test data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Changing the input-output data into deviation form:
ua_d=ua-ua(1);
ya_d=ya-ya(1);

%Defining the columns of the X matrix
x1=-ya_d(1:end-1);
x2=[0;-ya_d(1:end-2)];
x3=[0;ua_d(1:end-2)];
x4=[0;0; ua_d(1:end-3)];

%Constructing the X and Y matrix
X=[x1 x2 x3 x4];
Y=ya_d(2:end);

%Number of observations used in regression
N=length(Y);

%Number of unknown parameters to be estimated
p=4;

%Parameter estimates
beta=inv(X'*X)*X'*Y

%Predicted outputs
Ya_hat=X*beta;

%Prediction errors
Pred_Error=Y-Ya_hat;

%Prediction Variance
Sigma2=1/(N-p)*sum(Pred_Error.^2)

%Parameter Covariance
Cov_Beta=inv(X'*X)*Sigma2

%Comparing the fit and plotting the residuals
subplot(1,2,1);plot(1:1:N,Y,'r');hold on;plot(1:1:N,Ya_hat,'b');
xlabel('time');ylabel('output');
legend('Exp. data','Model pred.')

subplot(1,2,2);plot(1:1:N,Pred_Error,'r')
xlabel('time');ylabel('residuals');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%PART B: This section of the code estimates the model parameters using PRBS
%test data
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Changing the input-output data into deviation form:
ub_d=ub-ub(1);
yb_d=yb-yb(1);

%Defining the columns of the X matrix
x1=-yb_d(1:end-1);
x2=[0;-yb_d(1:end-2)];
x3=[0;ub_d(1:end-2)];
x4=[0;0; ub_d(1:end-3)];

%Constructing the X and Y matrix
X=[x1 x2 x3 x4];
Y=yb_d(2:end);

%Number of observations used in regression
N=length(Y);

%Number of unknown parameters to be estimated
p=4;

%Parameter estimates
beta=inv(X'*X)*X'*Y

%Predicted outputs
Yb_hat=X*beta;

%Prediction errors
Pred_Error=Y-Yb_hat;

%Prediction Variance
Sigma2=1/(N-p)*sum(Pred_Error.^2)

%Parameter Covariance
Cov_Beta=inv(X'*X)*Sigma2

%Comparing the fit and plotting the residuals
subplot(1,2,1);plot(1:1:N,Y,'r');hold on;plot(1:1:N,Yb_hat,'b');
xlabel('time');ylabel('output');
legend('Exp. data','Model pred.')

subplot(1,2,2);plot(1:1:N,Pred_Error,'r')
xlabel('time');ylabel('residuals');

